@@ -53,7 +53,7 @@ class Expression:
         field = self.args.get(key)
         if isinstance(field, str):
             return field
-        if isinstance(field, (Identifier, Literal)):
+        if isinstance(field, (Identifier, Literal, Var)):
             return field.this
         return ""
 
@@ -71,6 +71,9 @@ class Expression:
     def alias_or_name(self):
         return self.alias or self.name
 
+    def __deepcopy__(self, memo):
+        return self.__class__(**deepcopy(self.args))
+
     def copy(self):
         new = deepcopy(self)
         for item, parent, _ in new.bfs():
@@ -136,10 +139,8 @@ class Expression:
 
     def find_ancestor(self, *expression_types):
         ancestor = self.parent
-        while not isinstance(ancestor, expression_types):
+        while ancestor and not isinstance(ancestor, expression_types):
             ancestor = ancestor.parent
-            if not ancestor:
-                return None
         return ancestor
 
     def walk(self, bfs=True):
@@ -156,7 +157,7 @@ class Expression:
         if bfs:
             yield from self.bfs()
         else:
-            yield from self.dfs(self.parent)
+            yield from self.dfs()
 
     def dfs(self, parent=None, key=None, prune=None):
         """
@@ -166,6 +167,7 @@ class Expression:
         Returns:
             the generator object.
         """
+        parent = parent or self.parent
         yield self, parent, key
         if prune and prune(self, parent, key):
             return
@@ -215,9 +217,9 @@ class Expression:
         """
         Returns unnested operands as a list.
         """
-        return [arg.unnest() for arg in self.args.values()]
+        return [arg.unnest() for arg in self.args.values() if arg]
 
-    def flatten(self):
+    def flatten(self, unnest=True):
         """
         Returns a generator which yields child nodes who's parents are the same class.
 
@@ -227,7 +229,10 @@ class Expression:
             prune=lambda n, p, *_: p and not isinstance(n, self.__class__)
         ):
             if not isinstance(node, self.__class__):
-                yield node
+                yield node.unnest() if unnest else node
+
+    def __str__(self):
+        return self.sql()
 
     def __repr__(self):
         return self.to_s()
@@ -289,7 +294,10 @@ class Expression:
 
         if new_node is None:
             raise ValueError("A transformed node cannot be None")
-        if not isinstance(new_node, Expression) or new_node is not node:
+        if not isinstance(new_node, Expression):
+            return new_node
+        if new_node is not node:
+            new_node.parent = node.parent
             return new_node
 
         replace_children(
@@ -314,9 +322,10 @@ class Expression:
         if not self.parent:
             return
 
-        replace_children(
-            self.parent, lambda child: expressions if child is self else child
-        )
+        parent = self.parent
+        self.parent = None
+
+        replace_children(parent, lambda child: expressions if child is self else child)
 
     def assert_is(self, type_):
         """
@@ -463,7 +472,7 @@ class Column(Condition):
 
     @property
     def table(self):
-        return self.args.get("table")
+        return self.text("table")
 
 
 class ColumnDef(Expression):
@@ -511,13 +520,12 @@ class Identifier(Expression):
     arg_types = {"this": True, "quoted": False}
 
     def __eq__(self, other):
-        return (
-            isinstance(other, self.__class__)
-            and (self.this or "").upper() == (other.this or "").upper()
+        return isinstance(other, self.__class__) and _norm_arg(self.this) == _norm_arg(
+            other.this
         )
 
     def __hash__(self):
-        return hash((self.key, self.this.upper()))
+        return hash((self.key, self.this.lower()))
 
 
 class Insert(Expression):
@@ -554,11 +562,11 @@ class Literal(Condition):
         return (
             isinstance(other, Literal)
             and self.this == other.this
-            and self.args.get("is_string") == other.args.get("is_string")
+            and self.args["is_string"] == other.args["is_string"]
         )
 
     def __hash__(self):
-        return hash((self.key, self.this, self.args.get("is_string")))
+        return hash((self.key, self.this, self.args["is_string"]))
 
     @classmethod
     def number(cls, number):
@@ -580,6 +588,47 @@ class Literal(Condition):
 class Join(Expression):
     arg_types = {"this": True, "on": False, "side": False, "kind": False}
 
+    @property
+    def kind(self):
+        return self.text("kind").upper()
+
+    def on(self, *expressions, append=True, dialect=None, parser_opts=None, copy=True):
+        """
+        Append to or set the ON expressions.
+
+        Example:
+            >>> import sqlglot
+            >>> sqlglot.parse_one("JOIN x", into=Join).on("y = 1").sql()
+            'JOIN x ON y = 1'
+
+        Args:
+            *expressions (str or Expression): the SQL code strings to parse.
+                If an `Expression` instance is passed, it will be used as-is.
+                Multiple expressions are combined with an AND operator.
+            append (bool): if `True`, AND the new expressions to any existing expression.
+                Otherwise, this resets the expression.
+            dialect (str): the dialect used to parse the input expressions.
+            parser_opts (dict): other options to use to parse the input expressions.
+            copy (bool): if `False`, modify this expression instance in-place.
+
+        Returns:
+            Join: the modified join expression.
+        """
+        join = _apply_conjunction_builder(
+            *expressions,
+            instance=self,
+            arg="on",
+            append=append,
+            dialect=dialect,
+            parser_opts=parser_opts,
+            copy=copy,
+        )
+
+        if join.kind == "CROSS":
+            join.set("kind", None)
+
+        return join
+
 
 class Lateral(Expression):
     arg_types = {"this": True, "outer": False, "table": False, "columns": False}
@@ -756,14 +805,7 @@ class Values(Expression):
 
 
 class Var(Expression):
-    def __eq__(self, other):
-        return (
-            isinstance(other, self.__class__)
-            and (self.this or "").upper() == (other.this or "").upper()
-        )
-
-    def __hash__(self):
-        return hash((self.key, self.this.upper()))
+    pass
 
 
 class Schema(Expression):
@@ -1884,14 +1926,13 @@ class Year(Func):
 
 def _norm_args(expression):
     return {
-        k: _norm_arg(arg) if not isinstance(arg, list) else [_norm_arg(a) for a in arg]
+        k: [_norm_arg(a) for a in arg] if isinstance(arg, list) else _norm_arg(arg)
         for k, arg in expression.args.items()
     }
 
 
 def _norm_arg(arg):
-    arg = arg or ""
-    return arg.upper() if isinstance(arg, str) else arg
+    return arg.lower() if isinstance(arg, str) else arg
 
 
 def _all_functions():
@@ -2033,7 +2074,7 @@ def _apply_conjunction_builder(
     *expressions,
     instance,
     arg,
-    into,
+    into=None,
     append=True,
     copy=True,
     dialect=None,
@@ -2047,11 +2088,11 @@ def _apply_conjunction_builder(
 
     existing = inst.args.get(arg)
     if append and existing is not None:
-        expressions = [existing.this] + list(expressions)
+        expressions = [existing.this if into else existing] + list(expressions)
 
     node = and_(*expressions, dialect=dialect, **(parser_opts or {}))
 
-    inst.set(arg, into(this=node))
+    inst.set(arg, into(this=node) if into else node)
     return inst
 
 
@@ -2073,6 +2114,50 @@ def _wrap_operator(expression):
     return expression
 
 
+def select(*expressions, dialect=None, **opts):
+    """
+    Initializes a syntax tree from one or multiple SELECT expressions.
+
+    Example:
+        >>> select("col1", "col2").from_("tbl").sql()
+        'SELECT col1, col2 FROM tbl'
+
+    Args:
+        *expressions (str or Expression): the SQL code string to parse as the expressions of a
+            SELECT statement. If an Expression instance is passed, this is used as-is.
+        dialect (str): the dialect used to parse the input expressions (in the case that an
+            input expression is a SQL string).
+        **opts: other options to use to parse the input expressions (again, in the case
+            that an input expression is a SQL string).
+
+    Returns:
+        Select: the syntax tree for the SELECT statement.
+    """
+    return Select().select(*expressions, dialect=dialect, parser_opts=opts)
+
+
+def from_(*expressions, dialect=None, **opts):
+    """
+    Initializes a syntax tree from a FROM expression.
+
+    Example:
+        >>> from_("tbl").select("col1", "col2").sql()
+        'SELECT col1, col2 FROM tbl'
+
+    Args:
+        *expressions (str or Expression): the SQL code string to parse as the FROM expressions of a
+            SELECT statement. If an Expression instance is passed, this is used as-is.
+        dialect (str): the dialect used to parse the input expression (in the case that the
+            input expression is a SQL string).
+        **opts: other options to use to parse the input expressions (again, in the case
+            that the input expression is a SQL string).
+
+    Returns:
+        Select: the syntax tree for the SELECT statement.
+    """
+    return Select().from_(*expressions, dialect=dialect, parser_opts=opts)
+
+
 def condition(expression, dialect=None, **opts):
     """
     Initialize a logical condition expression.
@@ -2206,7 +2291,7 @@ def alias_(expression, alias, table=False, dialect=None, quoted=None, **opts):
             If an Expression instance is passed, this is used as-is.
         alias (str or Identifier): the alias name to use. If the name has
             special charachters it is quoted.
-        alias (boolean): create a table alias, default false
+        table (boolean): create a table alias, default false
         dialect (str): the dialect used to parse the input expression.
         **opts: other options to use to parse the input expressions.
 
@@ -2246,6 +2331,21 @@ def subquery(expression, alias=None, dialect=None, **opts):
     return Select().from_(expression, dialect=dialect, **opts)
 
 
+def column(col, table=None, quoted=None):
+    """
+    Build a Column.
+    Args:
+        col (str or Expression): column name
+        table (str or Expression): table name
+    Returns:
+        Column: column instance
+    """
+    return Column(
+        this=to_identifier(col, quoted=quoted),
+        table=to_identifier(table, quoted=quoted),
+    )
+
+
 def replace_children(expression, fun):
     """
     Replace children of an expression with the result of a lambda fun(child) -> exp.
@@ -2283,9 +2383,7 @@ def column_table_names(expression):
     Returns:
         list: A list of unique names
     """
-    return list(
-        dict.fromkeys(column.text("table") for column in expression.find_all(Column))
-    )
+    return list(dict.fromkeys(column.table for column in expression.find_all(Column)))
 
 
 TRUE = Boolean(this=True)
