@@ -15,7 +15,7 @@ def decorrelate_subqueries(expression):
         >>> import sqlglot
         >>> expression = sqlglot.parse_one("SELECT * FROM x AS x WHERE (SELECT y.a FROM y AS y WHERE x.a = y.a) = 1 ")
         >>> decorrelate_subqueries(expression).sql()
-        'SELECT * FROM x AS x JOIN (SELECT y.a FROM y AS y WHERE TRUE GROUP BY y.a) AS "_d_0" ON _d_0.a = x.a AND ("_d_0".a) = 1 WHERE TRUE'
+        'SELECT * FROM x AS x JOIN (SELECT y.a FROM y AS y WHERE TRUE GROUP BY y.a) AS "_d_0" ON "_d_0"."a" = x.a AND ("_d_0".a) = 1 WHERE TRUE'
 
     Args:
         expression (sqlglot.Expression): expression to decorrelated
@@ -54,7 +54,9 @@ def decorrelate_subqueries(expression):
                 select.select(internal, copy=False)
 
             alias = f"_d_{next(sequence)}"
-            on = [f"{alias}.{internal.text('this')} = {column.sql()}"]
+            on = exp.and_(f"\"{alias}\".\"{internal.text('this')}\" = {column.sql()}")
+
+            predicate = select.find_ancestor(*exp.PREDICATES, exp.Exists)
 
             eq.replace(exp.TRUE)
             select.replace(
@@ -64,21 +66,19 @@ def decorrelate_subqueries(expression):
                 )
             )
 
-            predicate = select.find_ancestor(*exp.PREDICATES, exp.Exists)
-
             if predicate:
                 predicate.replace(exp.TRUE)
 
             if isinstance(predicate, exp.Exists):
                 select = select.select(internal, append=False)
             elif predicate:
-                on.append(predicate.sql())
+                on = exp.and_(on, predicate)
 
-            select = select.group_by(internal)
+            select = select.group_by(internal, copy=False)
 
             scope.parent.expression.join(
                 select,
-                on=" AND ".join(on),
+                on=on,
                 join_alias=alias,
                 copy=False,
             )
