@@ -2,6 +2,7 @@ import itertools
 
 import sqlglot
 import sqlglot.expressions as exp
+from sqlglot.optimizer.simplify import simplify
 from sqlglot.optimizer.scope import traverse_scope
 
 
@@ -21,6 +22,7 @@ def eliminate_subqueries(expression):
     Returns:
         sqlglot.Expression: qualified expression
     """
+    expression = simplify(expression)
     queries = {}
 
     for scope in traverse_scope(expression):
@@ -34,12 +36,14 @@ def eliminate_subqueries(expression):
             continue
 
         alias = f"_e_{next(sequence)}"
-        expression.with_(alias, as_=query.copy(), copy=False)
+
         for dup in duplicates:
             parent = dup.parent
             if isinstance(parent, exp.Subquery):
-                parent.replace(exp.alias_(alias, parent.alias_or_name))
+                parent.replace(exp.alias_(alias, parent.alias_or_name, table=True))
             elif isinstance(parent, exp.Union):
                 dup.replace(sqlglot.select("*").from_(alias))
 
+        expression.with_(alias, as_=query, copy=False)
+
     return expression
