@@ -53,6 +53,10 @@ class Scope:
         self.subquery_scopes = []
         self.union = None
 
+        self._selected_sources = None
+        self._columns = None
+        self._external_columns = None
+
     def branch(self, expression, scope_type, add_sources=None, **kwargs):
         """Branch from the current scope to a new, inner scope"""
         sources = copy(self.sources)
@@ -131,43 +135,58 @@ class Scope:
             list[exp.Column]: Column instances in this scope, plus any
                 Columns that reference this scope from correlated subqueries.
         """
-        columns = self._find_in_scope(
-            lambda n: isinstance(n, exp.Column) and not isinstance(n.this, exp.Star)
-        )
-        return [
-            c
-            for c in columns + [c for c, _ in self.columns_from_subqueries]
-            if not self._is_reference_to_named_select(c)
-        ]
+        if self._columns is None:
+            columns = self._find_in_scope(
+                lambda n: isinstance(n, exp.Column) and not isinstance(n.this, exp.Star)
+            )
+
+            external_columns = [
+                column
+                for scope in self.subquery_scopes
+                for column in scope.external_columns
+            ]
+
+            self._columns = [
+                c
+                for c in columns + external_columns
+                if not self._is_reference_to_named_select(c)
+            ]
+        return self._columns
 
     @property
     def selected_sources(self):
         """
-        Mapping of sources that are actually selected from in this scope.
+        Mapping of nodes and sources that are actually selected from in this scope.
 
         That is, all tables in a schema are selectable at any point. But a
         table only becomes a selected source if it's included in a FROM or JOIN clause.
 
         Returns:
-            dict[str, exp.Table|Scope]: selected sources
+            dict[str, (exp.Table|exp.Subquery, exp.Table|Scope)]: selected sources and nodes
         """
-        referenced_names = []
+        if self._selected_sources is None:
+            referenced_names = []
 
-        for table in self.tables:
-            if isinstance(table.parent, exp.Alias):
-                referenced_names.append(table.parent.alias)
-            else:
-                referenced_names.append(table.name)
-        for derived_table in self.derived_tables:
-            referenced_names.append(derived_table.alias)
+            for table in self.tables:
+                referenced_names.append(
+                    (
+                        table.parent.alias
+                        if isinstance(table.parent, exp.Alias)
+                        else table.name,
+                        table,
+                    )
+                )
+            for derived_table in self.derived_tables:
+                referenced_names.append((derived_table.alias, derived_table.this))
 
-        result = {}
+            result = {}
 
-        for name in referenced_names:
-            if name in self.sources:
-                result[name] = self.sources[name]
+            for name, node in referenced_names:
+                if name in self.sources:
+                    result[name] = (node, self.sources[name])
 
-        return result
+            self._selected_sources = result
+        return self._selected_sources
 
     @property
     def selects(self):
@@ -186,22 +205,6 @@ class Scope:
             return []
         return self.expression.selects
 
-    @property
-    def columns_from_subqueries(self):
-        """
-        Columns referenced by correlated subqueries.
-
-        Returns:
-            list[tuple[exp.Column, Scope]]: List of Column instances from
-                child scopes that reference sources in this scope, along with the child
-                scope instance itself.
-        """
-        result = []
-        for scope in self.subquery_scopes:
-            for column in scope.external_columns:
-                result.append((column, scope))
-        return result
-
     @property
     def external_columns(self):
         """
@@ -211,7 +214,11 @@ class Scope:
             list[exp.Column]: Column instances that don't reference
                 sources in the current scope.
         """
-        return [c for c in self.columns if c.text("table") not in self.sources]
+        if self._external_columns is None:
+            self._external_columns = [
+                c for c in self.columns if c.table not in self.selected_sources
+            ]
+        return self._external_columns
 
     def source_columns(self, source_name):
         """
@@ -222,9 +229,7 @@ class Scope:
         Returns:
             list[exp.Column]: Column instances that reference `source_name`
         """
-        return [
-            column for column in self.columns if column.text("table") == source_name
-        ]
+        return [column for column in self.columns if column.table == source_name]
 
     @property
     def is_subquery(self):
@@ -234,7 +239,7 @@ class Scope:
     @property
     def is_correlated_subquery(self):
         """Determine if this scope is a correlated subquery"""
-        return self.is_subquery and self.external_columns
+        return bool(self.is_subquery and self.external_columns)
 
     def rename_source(self, old_name, new_name):
         """Rename a source in this scope"""
@@ -243,8 +248,8 @@ class Scope:
 
     def _is_reference_to_named_select(self, column):
         """Determine if column is a reference to a SELECT output column"""
-        table_name = column.text("table")
-        column_name = column.text("this")
+        table_name = column.table
+        column_name = column.name
 
         # Expression.named_selects also includes unaliased columns.
         # In this case, we want to be sure to only include selects that are aliased.
@@ -355,7 +360,7 @@ def _traverse_derived_tables(derived_tables, scope, scope_type):
 def _add_table_sources(scope):
     sources = {}
     for table in scope.tables:
-        table_name = table.text("this")
+        table_name = table.name
 
         if isinstance(table.parent, exp.Alias):
             source_name = table.parent.alias
@@ -375,11 +380,13 @@ def _add_table_sources(scope):
 
 def _traverse_subqueries(scope):
     for subquery in scope.subqueries:
+        top = None
         for child_scope in _traverse_scope(
             scope.branch(subquery, scope_type=ScopeType.SUBQUERY)
         ):
             yield child_scope
-            scope.subquery_scopes.append(child_scope)
+            top = child_scope
+        scope.subquery_scopes.append(top)
 
 
 def _bfs_until_next_scope(expression):
